@@ -1,3 +1,4 @@
+from copy import deepcopy
 import torch
 import inspect
 from typing import Callable, Union
@@ -129,16 +130,54 @@ class Trainer(PLTrainer):
         ckpt_trainer.datamodule = self.datamodule
         return ckpt_trainer
 
-    def save_best_mode(self, path='best_model.pth'):
+    def restore_best(self):
+        """
+        restore best checkpoint
+        """
         model = self.task_module.model
         loss = self.task_module.loss_fn
         opt = self.task_module.opt
-        task_model = self.task_module.load_from_checkpoint(self.checkpoint_callback.best_model_path, 
-                                                           model=model, loss=loss, optimizer=opt)
-        torch.save(task_model.model.state_dict(), path)
+        self.task_module = self.task_module.load_from_checkpoint(self.checkpoint_callback.best_model_path, 
+                                                            model=model, loss=loss, optimizer=opt)
+        return self.task_module.model
 
-    @staticmethod
-    def load_state_dict(model, path):
-        """load state_dict for torch model"""
-        model.load_state_dict(torch.load(path))
-        return model
\ No newline at end of file
+    def restore_best_k(self):
+        """
+        restore best k checkpoints. k is value of `save_top_k` argument in the `ModelCheckpoint` callback.
+        """
+        best_ckp_paths = self.checkpoint_callback.best_k_models.keys()
+        best_models = []
+        loss = self.task_module.loss_fn
+        opt = self.task_module.opt
+        model =self.task_module.model
+        for ckp_path in best_ckp_paths:
+            self.task_module.load_from_checkpoint(ckp_path, model=model, loss=loss, optimizer=opt)
+            best_models.append(deepcopy(self.task_module.model))
+        return best_models
+
+
+    def save_state_dict(self, path='best_model.pth', mode='best'):
+        """
+        Save state_dict of pytorch model.
+        Args:
+            path: path and filename of the state_dict file.
+            mode: 'best' for the best model and others for current model.
+        """
+        if mode == 'best':
+            model = self.task_module.model
+            loss = self.task_module.loss_fn
+            opt = self.task_module.opt
+            task = self.task_module.load_from_checkpoint(self.checkpoint_callback.best_model_path, 
+                                                            model=model, loss=loss, optimizer=opt)
+            torch.save(task.model.state_dict(), path)
+        else:
+            torch.save(self.task_module.model.state_dict(), path)
+
+    def load_state_dict(self, path='best_model.pth', model=None):
+        """load state_dict for pytorch model"""
+        if model:
+            model.load_state_dict(torch.load(path))
+        else:
+            self.task_module.model.load_state_dict(torch.load(path))
+            model = self.task_module.model
+        return model
